/* arquivo.cpp - biblioteca para manuseamento do arquivo da agenda */

#include <stdio.h>
#include <iostream.h>
#include <process.h>
#include <stdlib.h>
#include <string.h>

#include "arquivo.h"

/*--------------------------------------------------------------------------*\
 | disco::disco - Contrutor da classe disco, inicializa os campos da classe.|
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Nada.                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

disco::disco (void)

{

  fp_arq_base = NULL;

  cpz_identif_arq_base = NULL;

}

/*--------------------------------------------------------------------------*\
 | disco::disco - Contrutor da classe disco, abre os arquivos de leitura    |
 |                (arquivo de indice e da base de dados.                    |
 |                                                                          |
 | Recebe - Nome do arquivo de indice e o da base.                          |
 |                                                                          |
 | Retorna - Nada.                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

disco::disco (const char *cpz_nome_arq_base)

{

  if ((fp_arq_base = fopen (cpz_nome_arq_base, "r+b")) == NULL) {
    cerr << "Nao consegui abrir o arquivo " << cpz_nome_arq_base << "\n";
    exit (EXIT_FAILURE);
  }

  cpz_identif_arq_base = new char [strlen (cpz_nome_arq_base) + 1];
  strcpy (cpz_identif_arq_base, cpz_nome_arq_base);

}

/*--------------------------------------------------------------------------*\
 | disco::crie_arquivos - Constroi entradas no diretorio para o arquivo de  |
 |                        indice e para o arquivo de base.                  |
 |                                                                          |
 | Recebe - O nome do arquivo de indice e o nome do arquivo de base.        |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void disco::crie_arquivos (const char *cpz_nome_arq_base)
{

  if ((fp_arq_base = fopen (cpz_nome_arq_base, "wb")) == NULL) {
    cerr << "Nao consegui abrir o arquivo " << cpz_nome_arq_base << "\n";
    exit (EXIT_FAILURE);
  }

  fclose (fp_arq_base);

  if ((fp_arq_base = fopen (cpz_nome_arq_base, "r+b")) == NULL) {
    cerr << "Nao consegui abrir o arquivo " << cpz_nome_arq_base << "\n";
    exit (EXIT_FAILURE);
  }

  cpz_identif_arq_base = new char [strlen (cpz_nome_arq_base) + 1];
  strcpy (cpz_identif_arq_base, cpz_nome_arq_base);

}

/*--------------------------------------------------------------------------*\
 | disco::leia_arquivo_base - executa a leitura de um registro no arquivo   |
 |                            da base de dados.                             |
 |                                                                          |
 | Recebe - Um buffer contendo o registro, o seu tamanho e a posicao que em |
 |          que se encontra.                                                |
 |                                                                          |
 | Retorna - O registro lido.                                               |
\*--------------------------------------------------------------------------*/

void disco::leia_buffer (char *cpz_buffer, const int i_tam,
			   const int i_posic)
{

  fseek (fp_arq_base, (long) i_tam * (long) (i_posic - 1), SEEK_SET);

  if (fread (cpz_buffer, i_tam, 1, fp_arq_base) == 0) {
    cerr << "Erro no arquivo " << cpz_identif_arq_base << "\n";
    exit (EXIT_FAILURE);
  }

}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_base - grava o registro de base na primeira posicao     |
 |                         valida do arquivo, retornando entao a posicao    |
 |                         em que foi colocado o registro.                  |
 |                                                                          |
 | Recebe - Um buffer com o registro e um inteiro com o tamanho do mesmo.   |
 |                                                                          |
 | Retorna - Um inteiro que indica a posicao no arquivo em que foi colocado |
 |           o registro.                                   (Sobrecarregada) |
\*--------------------------------------------------------------------------*/


int disco::grave_buffer (char *cpz_buffer, const int i_tam)

{

  char *cpz_buf_verif = new char [i_tam + 1]; // buffer temporario

  int i_lidos;

  long l_posic;


  fseek (fp_arq_base, 0L, SEEK_SET);

  do {

    l_posic = ftell (fp_arq_base);

    i_lidos = fread (cpz_buf_verif, i_tam, 1, fp_arq_base);

  } while (strncmp (cpz_buf_verif, DELETADO, strlen (DELETADO)) &&
	 i_lidos != 0);

  delete cpz_buf_verif;

  if (i_lidos != 0)                               // se ha registro deletado
    fseek (fp_arq_base, (long) -i_tam, SEEK_CUR); //   grave nele.

  fwrite (cpz_buffer, i_tam, 1, fp_arq_base);

  l_posic /= i_tam;

  return ((int) l_posic + 1);

}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_base - grava o registro de base na posicao indicada.    |
 |                                                                          |
 | Recebe - Um buffer com o registro e dois inteiros, um com a posicao em   |
 |          que se deve gravar o registro e o outro com o tamanho do mesmo. |
 |                                                                          |
 | Retorna - Nada.                                                          |
 |                                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

void disco::grave_buffer (char *cpz_buffer, const int i_tam,
			    const int i_posicao)
{

  fseek (fp_arq_base, (long) (i_posicao - 1) * (long) i_tam, SEEK_SET);

  fwrite (cpz_buffer, i_tam, 1, fp_arq_base);

}

/*--------------------------------------------------------------------------*\
 | disco::delete_reg_arq_base - marca o registro base como deletado.        |
 |                                                                          |
 | Recebe - O tamanho de cada registro e a posicao do registro a ser        |
 |          deletado.                                                       |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void disco::delete_reg_arq_base (const int i_tam, const int i_posic)

{

  char *cpz_buffer = new char [i_tam + 1];


  strcpy (cpz_buffer, DELETADO);

  for (int i = strlen (DELETADO) + 1; i < i_tam; i++)
    cpz_buffer [i] = ' ';
  cpz_buffer [i_tam + 1] = '\0';

  fseek (fp_arq_base, (long) i_tam * (long) (i_posic - 1), SEEK_SET);

  if (fwrite (cpz_buffer, i_tam, 1, fp_arq_base) == 0) {
    delete cpz_buffer;
    cerr << "Erro no arquivo " << cpz_identif_arq_base << "\n";
    exit (EXIT_FAILURE);
  }

  delete cpz_buffer;

}

/*--------------------------------------------------------------------------*\
 | disco::prepara_busca - Inicializa o ponteiro do arquivo.                 |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/


void disco::prepara_busca (void)
{
  fseek (fp_arq_base, 0L, SEEK_SET);
}


/*--------------------------------------------------------------------------*\
 | disco::procura_buffer - Executa uma busca no aquivo, procurando um re-   |
 |                         gistro que possua um substring igual a chave     |
 |                         desejada.                                        |
 | Recebe - O tamanho do registro no arquivo, a posicao do inicio da chave  |
 |          no registro, o tamanho da chave e a propria chave.              |
 |                                                                          |
 | Retorna - Caso a funcao encontre o registro entao ela retorna VERDADEIRO,|
 |           juntamente com o buffer preenchido e a posicao do registro no  |
 |           arquivo. Caso contrario ela retorna FALSO.                     |
\*--------------------------------------------------------------------------*/

int disco::procura_buffer (char *buffer, const int i_tamanho,
			   const int i_comeco, const int i_tamanho_chave,
			   char *cpz_chave, int &i_posic)
{

  char *buffer_temp = new char [i_tamanho + 1],
       *chave_proc = new char [i_tamanho + 1];

  while (fread (buffer_temp, i_tamanho, 1, fp_arq_base) != 0) {
    strncpy (chave_proc, buffer_temp + i_comeco, i_tamanho_chave);
    chave_proc [i_tamanho_chave] = '\0';
    if (strstr (strupr(chave_proc), strupr(cpz_chave)) != NULL){
      strncpy (buffer, buffer_temp, i_tamanho);
      i_posic = ftell (fp_arq_base)/i_tamanho;
      return VERDADEIRO;
    }
  }
  i_posic = TERMINAR;
  return FALSO;
}

/*--------------------------------------------------------------------------*\
 | disco::leia_arq_base - Le um string do arquivo base.                     |
 |                                                                          |
 | Recebe - O formato de leitura.                                           |
 |                                                                          |
 | Retorna - O numero de caracteres lidos, e o string.                      |
 |                                                        (Sobrecarregada)  |
\*--------------------------------------------------------------------------*/

int disco::leia_arq_base (const char *cpz_formato, const char *cpz_argumento){
  return fscanf (fp_arq_base, cpz_formato, cpz_argumento);
}

/*--------------------------------------------------------------------------*\
 | disco::leia_arq_base - Le um caracter do arquivo base.                   |
 |                                                                          |
 | Recebe - O formato de leitura.                                           |
 |                                                                          |
 | Retorna - O numero de caracteres lidos, e o caracter.                    |
 |                                                        (Sobrecarregada)  |
\*--------------------------------------------------------------------------*/
int disco::leia_arq_base (const char *cpz_formato, const char c_argumento){
  return fscanf (fp_arq_base, cpz_formato, c_argumento);
}

/*--------------------------------------------------------------------------*\
 | disco::leia_arq_base - Le um inteiro do arquivo base.                    |
 |                                                                          |
 | Recebe - O formato de leitura.                                           |
 |                                                                          |
 | Retorna - O numero de caracteres lidos, e o inteiro.                     |
 |                                                        (Sobrecarregada)  |
\*--------------------------------------------------------------------------*/
int disco::leia_arq_base (const char *cpz_formato, const int &i_argumento) {
  return fscanf (fp_arq_base, cpz_formato, &i_argumento);
}

/*--------------------------------------------------------------------------*\
 | disco::leia_arq_base - Le um float do arquivo base.                      |
 |                                                                          |
 | Recebe - O formato de leitura.                                           |
 |                                                                          |
 | Retorna - O numero de caracteres lidos, e o float.                       |
 |                                                        (Sobrecarregada)  |
\*--------------------------------------------------------------------------*/
int disco::leia_arq_base (const char *cpz_formato, const float &f_argumento){
  return fscanf (fp_arq_base, cpz_formato, f_argumento);
}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_base - grava um string no arquivo base.                 |
 |                                                                          |
 | Recebe - O formato de gravacao.                                          |
 |                                                                          |
 | Retorna - O numero de caracteres gravados.                               |
 |                                                        (Sobrecarregada)  |
\*--------------------------------------------------------------------------*/
int disco::grave_arq_base (const char *cpz_formato, const char *cpz_argumento){
  return fprintf (fp_arq_base, cpz_formato, cpz_argumento);
}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_base - grava um caracter no arquivo base.               |
 |                                                                          |
 | Recebe - O formato de gravacao.                                          |
 |                                                                          |
 | Retorna - O numero de caracteres gravados.                               |
 |                                                        (Sobrecarregada)  |
\*--------------------------------------------------------------------------*/
int disco::grave_arq_base (const char *cpz_formato, const char c_argumento){
  return fprintf (fp_arq_base, cpz_formato, c_argumento);
}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_base - grava um inteiro no arquivo base.                |
 |                                                                          |
 | Recebe - O formato de gravacao.                                          |
 |                                                                          |
 | Retorna - O numero de caracteres gravados.                               |
 |                                                        (Sobrecarregada)  |
\*--------------------------------------------------------------------------*/
int disco::grave_arq_base (const char *cpz_formato, const int i_argumento){
  return fprintf (fp_arq_base, cpz_formato, i_argumento);
}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_base - grava um float no arquivo base.                  |
 |                                                                          |
 | Recebe - O formato de gravacao.                                          |
 |                                                                          |
 | Retorna - O numero de caracteres gravados.                               |
 |                                                        (Sobrecarregada)  |
\*--------------------------------------------------------------------------*/
int disco::grave_arq_base (const char *cpz_formato, const float f_argumento){
  return fprintf (fp_arq_base, cpz_formato, f_argumento);
}

/*--------------------------------------------------------------------------*\
 | disco::~disco - fecha os arquivos e elimina o nome dos arquivos.         |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

disco::~disco (void)

{

  if (fp_arq_base != NULL)
    fclose (fp_arq_base);

  if (cpz_identif_arq_base != NULL)
    delete cpz_identif_arq_base;

};

