/* bibliot.cpp */

#include <conio.h>
#include <string.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include "arquivo.h"
#include "definic.h"
#include "tela.h"


/*--------------------------------------------------------------------------*\
 | cria_registro - aloca e inicializa o registro de acordo com o formato.   |
 |                                                                          |
 | Recebe - O formato do registro.                                          |
 |                                                                          |
 | Retorna - O registro criado.                                             |
\*--------------------------------------------------------------------------*/

void **cria_registro (formato_arquivo fmt)
{

  void **registro = (void **)malloc (fmt.n_campos * sizeof (void *));

  for (int i = 0; i < fmt.n_campos; i++) {
    switch (fmt.tipo_campo [i][0]) {
      case 'C':
      case 'c': {
	registro [i] = malloc (fmt.tamanho_campo [i] + 1);
	strcpy ((char *)registro [i], "");
	break;
      }
      case 'N':
      case 'n': {
	if (fmt.casas_dec [i] == 0) {
	  registro [i] = malloc (sizeof (long int));
	  *(long int *)registro [i] = 0;
	} else {
	  registro [i] = malloc (sizeof (float));
	  *(float *)registro [i] = 0.0;
	}
	break;
      }
    }
  }

  return registro;
}

/*--------------------------------------------------------------------------*\
 | zera_registro - inicializa as posicoes do registro de acordo com o forma-|
 |                 to.                                                      |
 |                                                                          |
 | Recebe - O registro e o formato.                                         |
 |                                                                          |
 | Retorna - O registro inicializado.                                       |
\*--------------------------------------------------------------------------*/
void zera_registro (void **registro, formato_arquivo fmt)
{

  for (int i = 0; i < fmt.n_campos; i++) {
    switch (fmt.tipo_campo [i][0]) {
      case 'C':
      case 'c': {
	strcpy ((char *)registro [i], "");
	break;
      }
      case 'N':
      case 'n': {
	if (fmt.casas_dec [i] == 0)
	  *(long int *)registro [i] = 0;
	else
	  *(float *)registro [i] = 0.0;
	break;
      }
    }
  }
}


/*--------------------------------------------------------------------------*\
 | destroi_registro - desaloca as posicoes do registro.                     |
 |                                                                          |
 | Recebe - O registro e o formato.                                         |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/
void destroi_registro (void **registro, formato_arquivo fmt)
{
  for (int i = 0; i < fmt.n_campos; i++)
    free (registro [i]);
  free (registro);
}


/*--------------------------------------------------------------------------*\
 | mostra_estrutura - Exibe o registro na tela e pergunta se e o desejado.  |
 |                                                                          |
 | Recebe - O titulo da tela, o formato e o registro.                       |
 |                                                                          |
 | Retorna - VERDADEIRO, se for o registro desejado, FALSO c.c..            |
\*--------------------------------------------------------------------------*/
int mostra_estrutura (char *cpz_titulo,
		     formato_arquivo fmt, void **registro)
{

  int linha = 0,
      comando;


  tela edicao (TELA_50_LINHAS, SISTEMA, cpz_titulo);

  do {
    edicao.escreve_esquerda (linha + 8, fmt.nome_campo [linha]);
    edicao.escreve (23, linha + 8, ":", 0);
    switch (fmt.tipo_campo [linha][0]) {
      case 'C':
      case 'c': {
	edicao.escreve (24, linha + 8, "%-s", (char *) registro [linha]);
	edicao.escreve (24 + fmt.tamanho_campo [linha], linha + 8, ":", 0);
	break;
      }
      case 'N':
      case 'n': {
	if (fmt.casas_dec [linha] == 0) {
	  edicao.escreve (24, linha + 8, "%ld",
			  *((long int *) registro [linha]));
	  edicao.escreve (24 + fmt.tamanho_campo [linha], linha + 8, ":", 0);
	} else {
	  char formato [10], tamanho [10], tamanho_real [10];

	  strcpy (formato, "%");
	  itoa (fmt.tamanho_campo [linha], tamanho, 10);
	  strcat (tamanho, ".");
	  strcat (tamanho, itoa (fmt.casas_dec [linha], tamanho_real, 10));
	  strcat (formato, tamanho);
	  strcat (formato, "f");
	  edicao.escreve (24, linha + 8, formato,
			  *((float *) registro [linha]));
	  edicao.escreve (24 + fmt.tamanho_campo [linha]
			  + fmt.casas_dec [linha] + 1,
			  linha + 8, ":", 0);
	}
	break;
      }
    }
  } while (++linha < fmt.n_campos);

  if (edicao.confirmacao ("E este o registro procurado?"))
    return VERDADEIRO;
  else
    return FALSO;

}

/*--------------------------------------------------------------------------*\
 | edita_estrutura - Exibe o registro na tela e permite a alteracao dos da- |
 |                   dos.                                                   |
 |                                                                          |
 | Recebe - O titulo da tela, o formato e o registro.                       |
 |                                                                          |
 | Retorna - VERDADEIRO, se as alteracoes forem confirmadas e FALSO se for  |
 |           pressionado ESC ou as alteracoes nao forem confirmadas.        |
\*--------------------------------------------------------------------------*/
int edita_estrutura (char *cpz_titulo,
		     formato_arquivo fmt, void **registro)
{

  int linha = 0,
      comando;


  tela edicao (TELA_50_LINHAS, SISTEMA, cpz_titulo);

  do {
    edicao.escreve_esquerda (linha + 8, fmt.nome_campo [linha]);
    edicao.escreve (23, linha + 8, ":", 0);
    switch (fmt.tipo_campo [linha][0]) {
      case 'C':
      case 'c': {
	edicao.escreve (24, linha + 8, "%-s", (char *) registro [linha]);
	edicao.escreve (24 + fmt.tamanho_campo [linha], linha + 8, ":", 0);
	break;
      }
      case 'N':
      case 'n': {
	if (fmt.casas_dec [linha] == 0) {
	  edicao.escreve (24, linha + 8, "%ld",
			  *((long int *) registro [linha]));
	  edicao.escreve (24 + fmt.tamanho_campo [linha], linha + 8, ":", 0);
	} else {
	  char formato [10], tamanho [10], tamanho_real [10];

	  strcpy (formato, "%");
	  itoa (fmt.tamanho_campo [linha], tamanho, 10);
	  strcat (tamanho, ".");
	  strcat (tamanho, itoa (fmt.casas_dec [linha], tamanho_real, 10));
	  strcat (formato, tamanho);
	  strcat (formato, "f");
	  edicao.escreve (24, linha + 8, formato,
			  *((float *) registro [linha]));
	  edicao.escreve (24 + fmt.tamanho_campo [linha]
			  + fmt.casas_dec [linha] + 1,
			  linha + 8, ":", 0);
	}
	break;
      }
    }
  } while (++linha < fmt.n_campos);

  linha = 0;

  edicao.poe_mensagem ("Edite os campos ou <ESC> para sair.");
  do {
    switch (fmt.tipo_campo [linha][0]) {
      case 'C':
      case 'c': {
	char temp [60];
	strcpy (temp, (char *) registro [linha]);
	comando = edicao.edita_campo (24, linha + 8,
				      temp,
				      fmt.tamanho_campo [linha]);
	strcpy ((char *) registro [linha], temp);
	break;
      }
      case 'N':
      case 'n': {
	if (fmt.casas_dec [linha] == 0) {
	  long int temp;

	  temp = *((long int *) registro [linha]);
	  comando = edicao.pega_numero (24, linha + 8,
					temp,
					fmt.tamanho_campo [linha]);
	  *((long int *) registro [linha]) = temp;
	} else {
	  float temp;

	  temp = *((float *) registro [linha]);
	  comando = edicao.pega_numero (24, linha + 8,
					temp,
					fmt.tamanho_campo [linha],
					fmt.casas_dec [linha]);
	  *((float *) registro [linha]) = temp;
	}
	break;
      }
    }

    switch (comando) {
      case SOBE: {
	if (--linha == -1) {
	  linha = 0;
	}
	break;
      }
      case SAI:
	return FALSO;
      case DESCE:
      default:
	if (++linha == fmt.n_campos) {
	  edicao.apaga_mensagem ();
	  if (edicao.confirmacao ("Confirma os dados acima?") == NAO)
	    linha = 0;
	  else
	    return VERDADEIRO;
	}
    }
  } while (linha < fmt.n_campos);
  edicao.apaga_mensagem ();
  return VERDADEIRO;
}





/*--------------------------------------------------------------------------*\
 | calcula_posicao - Calcula a posicao relativa de um campo no registro.    |
 |                                                                          |
 | Recebe - O formato, e o indice do campo no registro.                     |
 |                                                                          |
 | Retorna - O comeco do campo e o seu tamanho.                             |
\*--------------------------------------------------------------------------*/
void calcula_posicao (formato_arquivo fmt, int indice, int &i_comeco,
		      int &i_tamanho)
{

  i_comeco = 0;

  switch (fmt.tipo_campo [indice][0]) {
    case 'C':
    case 'c': {
      i_tamanho = fmt.tamanho_campo [indice];
      break;
    }
    case 'N':
    case 'n':  {
      if (fmt.casas_dec [indice] == 0)
	i_tamanho = fmt.tamanho_campo [indice];
      else {
	i_tamanho = fmt.tamanho_campo [indice] + fmt.casas_dec [indice] + 1;
      }
    }
  }

  for (int i = 0; i < indice; i++) {
    switch (fmt.tipo_campo [i][0]) {
      case 'C':
      case 'c': {
	i_comeco += fmt.tamanho_campo [i];
	break;
      }
      case 'N':
      case 'n':  {
	if (fmt.casas_dec [i] == 0)
	  i_comeco += fmt.tamanho_campo [i];
	else {
	  i_comeco += fmt.tamanho_campo [i] + fmt.casas_dec [i] + 1;
	}
	break;
      }
    }
  }
}

/*--------------------------------------------------------------------------*\
 | escolhe_chave - Solicita a escolha do campo que servira de chave para    |
 |                 pesquisa.                                                |
 |                                                                          |
 | Recebe - O formato.                                                      |
 |                                                                          |
 | Retorna - O indice da chave escolhida.                                   |
\*--------------------------------------------------------------------------*/
int escolhe_chave (formato_arquivo fmt)
{

  tela escolhe (TELA_50_LINHAS, SISTEMA, "ESCOLHA DA CHAVE");

  int i;
  for (i = 0; i < fmt.n_campos; i++) {
    char nome_campo [30];

    sprintf (nome_campo, "%2d - %-20s", i + 1, fmt.nome_campo [i]);
    escolhe.escreve_meio (i + 8, nome_campo);
  }

  escolhe.poe_mensagem ("Digite o numero do campo desejado como chave.");

  escolhe.escreve (10, i + 10, "Chave:  :", 0);
  int chave = 0;
  do {
    escolhe.pega_numero (16, i + 10, chave, 2);
  } while (chave < 1 || chave > fmt.n_campos);

  return chave - 1;

}



/*--------------------------------------------------------------------------*\
 | buffer_para_registro - Converte os bytes do buffer para o formato do re- |
 |                        gistro.                                           |
 |                                                                          |
 | Recebe - O formato e o buffer.                                           |
 |                                                                          |
 | Retorna - O registro com os dados.                                       |
\*--------------------------------------------------------------------------*/
void buffer_para_registro (formato_arquivo fmt, void **registro,
			   char *buffer)
{
  int comeco_campo, tamanho_campo;

  for (int i = 0; i < fmt.n_campos; i++) {
    calcula_posicao (fmt, i, comeco_campo, tamanho_campo);
    char temp [60];
    strncpy (temp, buffer + comeco_campo, tamanho_campo);
    temp [tamanho_campo] = '\0';  // finaliza o string

    switch (fmt.tipo_campo [i][0]) {
      case 'C':
      case 'c': {
	strcpy ((char *)registro [i], temp);
	break;
      }
      case 'N':
      case 'n':  {
	if (fmt.casas_dec [i] == 0)
	  *((long int *)registro [i]) = atol (temp);
	else {
	  *((float *)registro [i]) = atof (temp);
	}
	break;
      }
    }

  }

}


/*--------------------------------------------------------------------------*\
 | registro_para_buffer - Converte os campos do registro para um buffer.    |
 |                                                                          |
 | Recebe - O formato e o registro.                                         |
 |                                                                          |
 | Retorna - O buffer preenchido.                                           |
\*--------------------------------------------------------------------------*/
void registro_para_buffer (formato_arquivo fmt, char *buffer,
			   void **registro)
{
  int comeco_campo, tamanho_campo;

  strcpy (buffer, "");
  for (int i = 0; i < fmt.n_campos; i++) {
    char temp [60];
    switch (fmt.tipo_campo [i][0]) {
      case 'C':
      case 'c': {
	char formato_str [10], tamanho [10];

	strcpy (formato_str, "%-");
	itoa (fmt.tamanho_campo [i], tamanho, 10);
	strcat (formato_str, tamanho);
	strcat (formato_str, "s");
	sprintf (temp, formato_str, (char *)registro [i]);
	strcat (buffer, temp);
	break;
      }
      case 'N':
      case 'n':  {
	if (fmt.casas_dec [i] == 0){
	  char formato_str [10], tamanho [10];

	  strcpy (formato_str, "%");
	  itoa (fmt.tamanho_campo [i], tamanho, 10);
	  strcat (formato_str, tamanho);
	  strcat (formato_str, "ld");
	  sprintf (temp, formato_str, *((long int *)registro [i]));
	  strcat (buffer, temp);
       } else {
	  char formato_str [10], tamanho [10], tamanho_real [10];

	  strcpy (formato_str, "%");
	  itoa (fmt.tamanho_campo [i], tamanho, 10);
	  strcat (tamanho, ".");
	  strcat (tamanho, itoa (fmt.casas_dec [i], tamanho_real, 10));
	  strcat (formato_str, tamanho);
	  strcat (formato_str, "f");
	  sprintf (temp, formato_str, *((float *)registro [i]));
	  for (int k = strlen (temp); k < fmt.tamanho_campo [i] +
					  fmt.casas_dec [i] + 1; k++)
	    temp [k] = ' ';
	  temp [fmt.tamanho_campo [i] + fmt.casas_dec [i] + 1] = '\0';
	  strcat (buffer, temp);
	}
	break;
      }
    }

  }

}


/*--------------------------------------------------------------------------*\
 | procura_registro - Procura um registro no arquivo.                       |
 |                                                                          |
 | Recebe - O nome do arquivo, o formato e a chave a ser procurada.         |
 |                                                                          |
 | Retorna - Se achou retorna o registro e sua posicao no arquivo, senao    |
 |           retorna TERMINAR.                                              |
\*--------------------------------------------------------------------------*/
int procura_registro (char *nome, void **registro,
		      formato_arquivo fmt, int chave)
{
  tela procura (TELA_50_LINHAS, SISTEMA, "PROCURA REGISTRO");

  char campo [60];

  strcpy (campo, "");
  procura.escreve (5, 20, "Chave  :", 0);

  switch (fmt.tipo_campo [chave][0]) {
    case 'C':
    case 'c': {
      procura.escreve (5 + fmt.tamanho_campo [chave]
			 + fmt.casas_dec [chave], 20, ":", 0);
      procura.poe_mensagem ("Digite um substring do campo desejado");
      procura.edita_campo (13, 20, campo, fmt.tamanho_campo [chave]);
      break;
    }
    case 'N':
    case 'n':  {
      procura.poe_mensagem ("Digite o numero desejado");
      if (fmt.casas_dec [chave] == 0){
	procura.escreve (5 + fmt.tamanho_campo [chave]
			   + fmt.casas_dec [chave], 20, ":", 0);
	procura.edita_campo (13, 20, campo, fmt.tamanho_campo [chave]);

	char formato_str [10], tamanho [10];
	long int numero;

	numero = atol (campo);

	strcpy (formato_str, "%");
	itoa (fmt.tamanho_campo [chave], tamanho, 10);
	strcat (formato_str, tamanho);
	strcat (formato_str, "ld");

	sprintf (campo, formato_str, numero);
     } else {
	procura.escreve (5 + fmt.tamanho_campo [chave]
			   + fmt.casas_dec [chave] + 1, 20, ":", 0);
	procura.edita_campo (13, 20, campo, fmt.tamanho_campo [chave] +
					    fmt.casas_dec [chave] + 1);

	char formato_str [10], tamanho [10], tamanho_real [10];
	float numero;

	numero = atof (campo);

	strcpy (formato_str, "%");
	itoa (fmt.tamanho_campo [chave], tamanho, 10);
	strcat (tamanho, ".");
	strcat (tamanho, itoa (fmt.casas_dec [chave], tamanho_real, 10));
	strcat (formato_str, tamanho);
	strcat (formato_str, "f");

	sprintf (campo, formato_str, numero);
	for (int k = strlen (campo); k < fmt.tamanho_campo [chave] +
					fmt.casas_dec [chave] + 1; k++)
	  campo [k] = ' ';
	campo [fmt.tamanho_campo [chave] + fmt.casas_dec [chave] + 1] = '\0';
      }
      break;
    }
  }
  char nome_base [15];
  procura.apaga_mensagem ();

  strcpy (nome_base, nome);
  strcat (nome_base, EXTENSAO_DADOS);
  disco d_procura (nome_base);
  d_procura.prepara_busca ();

  char *buffer = new char [fmt.tamanho_registro + 1];

  int comeco, tamanho, posicao;
  calcula_posicao (fmt, chave, comeco, tamanho);
  int comando;
  do {
    comando = d_procura.procura_buffer (buffer, fmt.tamanho_registro,
				comeco, tamanho, campo, posicao);
    if (comando) {
      buffer_para_registro (fmt, registro, buffer);
      if (mostra_estrutura ("PROCURA REGISTRO", fmt, registro)) {
	delete buffer;
	return posicao;
      }
    }
  } while (comando);

  delete buffer;

  return TERMINAR;
}
