/* cria_fmt.cpp */

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include "definic.h"
#include "arquivo.h"
#include "tela.h"


/*--------------------------------------------------------------------------*\
 | zera_formato - Inicializa a estrutura do arquivo.                        |
 |                                                                          |
 | Recebe - O estrutura por referencia.                                     |
 |                                                                          |
 | Retorna - A estrutura inicializada.                                      |
\*--------------------------------------------------------------------------*/
void zera_formato (formato_arquivo *fmt)
{
  memset (fmt, 0, sizeof (formato_arquivo));
}

/*--------------------------------------------------------------------------*\
 | grava_formato - Grava o formato no arquivo de formato.                   |
 |                                                                          |
 | Recebe - O estrutura com o formato e o nome do arquivo.                  |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/
void grava_formato (const char *nome, formato_arquivo form_arq)
{
  disco arq_formato;

  arq_formato.crie_arquivos (nome);

  arq_formato.grave_arq_base ("%u\n", form_arq.n_campos);
  arq_formato.grave_arq_base ("%u\n", form_arq.tamanho_registro);
  for (int i = 0; i < form_arq.n_campos; i++){
    arq_formato.grave_arq_base ("%-1s\n", &form_arq.tipo_campo[i][0]);
    arq_formato.grave_arq_base ("%-20s\n", &form_arq.nome_campo[i][0]);
    arq_formato.grave_arq_base ("%u\n", form_arq.tamanho_campo[i]);
    arq_formato.grave_arq_base ("%u\n", form_arq.casas_dec[i]);
  }
}


/*--------------------------------------------------------------------------*\
 | le_formato - Le o arquivo de formato, na estrutura de formato.           |
 |                                                                          |
 | Recebe - O nome do arquivo.                                              |
 |                                                                          |
 | Retorna - O formato lido.                                                |
\*--------------------------------------------------------------------------*/
void le_formato (char *nome, formato_arquivo &form_arq)
{
  disco arq_formato (nome);

  arq_formato.leia_arq_base ("%u\n", form_arq.n_campos);
  arq_formato.leia_arq_base ("%u\n", form_arq.tamanho_registro);
  for (int i = 0; i < form_arq.n_campos; i++){
    arq_formato.leia_arq_base ("%1s\n", &form_arq.tipo_campo[i][0]);
    arq_formato.leia_arq_base ("%20s\n", &form_arq.nome_campo[i][0]);
    arq_formato.leia_arq_base ("%u\n", form_arq.tamanho_campo[i]);
    arq_formato.leia_arq_base ("%u\n", form_arq.casas_dec[i]);
  }
}

/*--------------------------------------------------------------------------*\
 | calcula_tam_registro - Calcula o tamanho do registro em bytes.           |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - O tamanho do registro no campo apropriado do formato.          |
\*--------------------------------------------------------------------------*/
void calcula_tam_registro (formato_arquivo &fmt)
{

  for (int i = 0; i < fmt.n_campos; i++) {
    switch (fmt.tipo_campo [i][0]) {
      case 'C':
      case 'c': {
	fmt.tamanho_registro += fmt.tamanho_campo [i];
	break;
      }
      case 'N':
      case 'n':  {
	if (fmt.casas_dec [i] == 0)
	  fmt.tamanho_registro += fmt.tamanho_campo [i];
	else {
	  fmt.tamanho_registro += fmt.tamanho_campo [i] +
				  fmt.casas_dec [i] + 1;
	}
	break;
      }
    }
  }
}

/*--------------------------------------------------------------------------*\
 | cria_formato - Permite a criacao de um novo formato de arquivo.          |
 |                                                                          |
 | Recebe - O nome do arquivo.                                              |
 |                                                                          |
 | Retorna - O novo formato e VERDADEIRO, caso o usuario confirme a criacao,|
 |           retorna SAI caso o usuario retorne com ESC.                    |
\*--------------------------------------------------------------------------*/
int cria_formato (const char *nome_arquivo, formato_arquivo &fmt)
{

  tela formato (TELA_50_LINHAS, SISTEMA, "FORMATACAO");

  int pos = 1,
      linha,
      comando;

  zera_formato (&fmt);
  do {
    switch (pos) {
      case 1: {
	do {
	  formato.escreve_esquerda (7, "Numero de campos do arquivo: ");
	  formato.poe_mensagem ("Digite um valor entre 1 e 30");
	  switch (formato.pega_numero (32, 7, fmt.n_campos, 2)) {
	    case SOBE:
	    case SAI:
	      return SAI;
	  };
	} while (fmt.n_campos > MAX_CAMPOS || fmt.n_campos < 1);
	formato.apaga_mensagem ();
	pos = 3;
	linha = 0;
	break;
      }
      case 2: {
	if (--linha == -1)
	  pos = 1;
	else
	  pos = 3;
	break;

      }
      case 3: {
	formato.escreve (5, linha + 10, "%u: ", linha + 1);
	formato.poe_mensagem ("Digite o nome do campo");
	formato.escreve (8, linha + 10, "Nome: ", 0);
	switch (formato.edita_campo (14, linha + 10,
				     fmt.nome_campo [linha], 20)) {
	  case SOBE: {
	    pos--;
	    break;
	  }
	  case SAI:
	    return SAI;
	  case DESCE:
	  default:
	    pos++;
	}
	for (int j = 0; j < strlen (fmt.nome_campo [linha]); j++)
	  if (fmt.nome_campo [linha][j] == ' ')
	    fmt.nome_campo [linha][j] = '_';
	formato.apaga_mensagem ();
	break;
      }
      case 4: {
	formato.poe_mensagem ("C = Caracter ; N = Numero");
	do {
	  formato.escreve (35, linha + 10, "Tipo : ", 0);
	  comando = formato.edita_campo (42, linha + 10,
		    fmt.tipo_campo[linha], 1);
	} while (fmt.tipo_campo[linha][0] != 'C' &&
		 fmt.tipo_campo[linha][0] != 'c' &&
		 fmt.tipo_campo[linha][0] != 'N' &&
		 fmt.tipo_campo[linha][0] != 'n');
	switch (comando) {
	  case SOBE: {
	    pos--;
	    break;
	  }
	  case SAI:
	    return SAI;
	  case DESCE:
	  default:
	    pos++;
	}
	formato.apaga_mensagem ();
	break;
      }
      case 5: {
	if (fmt.tipo_campo[linha][0] == 'N' ||
	    fmt.tipo_campo[linha][0] == 'n'){
	  formato.poe_mensagem ("Digite um numero entre 1 e 10.");
	  do{
	    formato.escreve (46, linha + 10, "Tamanho: ", 0);
	    comando = formato.pega_numero (55, linha + 10,
		      fmt.tamanho_campo [linha], 2);

	  } while (fmt.tamanho_campo[linha] > 10 ||
		   fmt.tamanho_campo[linha] < 1);
	  switch (comando) {
	    case SOBE: {
	      pos--;
	      break;
	    }
	    case SAI:
	      return SAI;
	    case DESCE:
	    default:
	      pos++;
	  }
	} else {
	  formato.poe_mensagem ("Digite um numero entre 1 e 50.");
	  do{
	    formato.escreve (46, linha + 10, "Tamanho: ", 0);
	    comando = formato.pega_numero (55, linha + 10,
		    fmt.tamanho_campo[linha], 2);
	  } while (fmt.tamanho_campo[linha] > 50 ||
		   fmt.tamanho_campo[linha] < 1);
	  switch (comando) {
	    case SOBE: {
	      pos--;
	      break;
	    }
	    case SAI:
	      return SAI;
	    case DESCE:
	    default:
	      pos++;
	  }
	}
	formato.apaga_mensagem ();
	break;
      }
      case 6: {
	if (fmt.tipo_campo[linha][0] == 'N' ||
	    fmt.tipo_campo[linha][0] == 'n') {
	  formato.poe_mensagem ("Digite um numero entre 0 e 10.");
	  do {
	    formato.escreve (60, linha + 10, "Casas Decimais: ", 0);
	    comando = formato.pega_numero (77, linha + 10,
		      fmt.casas_dec[linha], 2);
	  } while (fmt.casas_dec[linha] > 10 || fmt.casas_dec[linha] < 0);
	  switch (comando) {
	    case SOBE: {
	      pos--;
	      break;
	    }
	    case SAI:
	      return SAI;
	    case DESCE:
	    default:
	      pos++;
	  }
	  formato.apaga_mensagem ();
	}
	else
	  pos++;
	break;
      }
      case 7: {
	if (++linha < fmt.n_campos) {
	  pos = 3;
	} else
	  if (formato.confirmacao ("Os dados estao corretos?") == NAO)
	    pos = linha = 1;
	  else
	    pos = 8;
	break;

      }

    }
  } while (pos != 8);

  calcula_tam_registro (fmt);

  char nome_arquivo_fmt [15],
       nome_arquivo_bse [55];

  strcpy (nome_arquivo_fmt, nome_arquivo);
  strcpy (nome_arquivo_bse, nome_arquivo);
  strcat (nome_arquivo_fmt, EXTENSAO_FORMATO);
  strcat (nome_arquivo_bse, EXTENSAO_DADOS);
  grava_formato (nome_arquivo_fmt, fmt);
  disco base;
  base.crie_arquivos (nome_arquivo_bse);

  return VERDADEIRO;

}








