/* tela.cpp - executa o gerenciamento de tela */

#include <conio.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <bios.h>
#include <dos.h>
#include <math.h>

#include "tela.h"

/*--------------------------------------------------------------------------*\
 | tela::tela_25 - desenha a estrutura da tela para 25 linhas               |
 |                                                                          |
 | Recebe - O titulo do sistema e o titulo da tela.                         |
 |                                                                          |
 | Retorna - Nada.                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

void tela::tela_25 (const char *cpz_nome_sistema, const char *cpz_titulo)

{

  clrscr ();
  moldura (1, 1, 80, 5, MOLD_DUPLO_AMBAS);
  escreve_meio (2, cpz_nome_sistema);
  escreve_direita (4, cpz_titulo);

  moldura (1, 6, 80, 21, MOLD_SIMPLES);

  moldura (1, 22, 80, 24, MOLD_DUPLO_AMBAS);

}

/*--------------------------------------------------------------------------*\
 | tela::tela_50 - desenha a estrutura da tela para 50 linhas.              |
 |                                                                          |
 | Recebe - O titulo do sistema e o titulo da tela                          |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void tela::tela_50 (const char *cpz_nome_sistema, const char *cpz_titulo)
{
  int i;

  textmode (C4350);
  clrscr ();
  moldura (1, 1, 80, 5, MOLD_DUPLO_AMBAS);
  escreve_meio (3, cpz_nome_sistema);
  escreve_direita (4, cpz_titulo);

  moldura (1, 6, 80, 44, MOLD_SIMPLES);

  moldura (1, 45, 80, 49, MOLD_DUPLO_AMBAS);
}


/*--------------------------------------------------------------------------*\
 | tela::tela (construtor) desenha a estrutura da tela.                     |
 |                                                                          |
 | Recebe - O modo da tela (numero de linhas), o titulo do sistema e o      |
 |          titulo da tela                                                  |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/
tela::tela (int i_modo, const char *cpz_nome_sistema, const char *cpz_titulo)
{
  gettextinfo (&sti_modo_ant);

  if (i_modo == TELA_25_LINHAS) {
    tela_25 (cpz_nome_sistema, cpz_titulo);
    i_tipo_tela = TELA_25_LINHAS;
    i_linha_mensagem = 23;
  }
  else if (i_modo == TELA_50_LINHAS) {
    tela_50 (cpz_nome_sistema, cpz_titulo);
    i_tipo_tela = TELA_50_LINHAS;
    i_linha_mensagem = 47;
  }
  else
    i_tipo_tela = MODO_INVALIDO;

  cpz_titulo_tela = NULL;
}

/*--------------------------------------------------------------------------*\
 | tela::moldura - desenha uma modura no video.                             |
 |                                                                          |
 | Recebe - As coordenadas do video e o tipo da moldura.                    |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void tela::moldura (const int X1, const int Y1, const int X2, const int Y2,
		    const int i_tipo)

{
  int i_cont;

  gotoxy (X1, Y1);  /* Escolha caracter superior esquerdo.*/

  switch (i_tipo) {
    case 1: putch (201); break;
    case 2: putch (177); break;
    case 3: putch (214); break;
    case 4: putch (213); break;
    case 5: putch (176); break;
    case 6: putch (218); break;
    case 7: putch (178); break;
    case 8: putch (219); break;
  }

  gotoxy (X2, Y1);  /* Escolha caracter superior direito.*/

  switch (i_tipo) {
    case 1: putch (187); break;
    case 2: putch (177); break;
    case 3: putch (183); break;
    case 4: putch (184); break;
    case 5: putch (176); break;
    case 6: putch (191); break;
    case 7: putch (178); break;
    case 8: putch (219); break;
  }

  gotoxy (X1,Y2);  /* Escolha caracter inferior esquerdo.*/

  switch (i_tipo) {
    case 1: putch (200); break;
    case 2: putch (177); break;
    case 3: putch (211); break;
    case 4: putch (212); break;
    case 5: putch (176); break;
    case 6: putch (192); break;
    case 7: putch (178); break;
    case 8: putch (219); break;
  }

  gotoxy (X2, Y2);  /* Escolha caracter inferior direito.*/

  switch (i_tipo) {
    case 1: putch (188); break;
    case 2: putch (177); break;
    case 3: putch (189); break;
    case 4: putch (190); break;
    case 5: putch (176); break;
    case 6: putch (217); break;
    case 7: putch (178); break;
    case 8: putch (219); break;
  }

  for (i_cont = Y1 + 1; i_cont < Y2; i_cont++) {

    gotoxy (X1, i_cont);  /* Linha vertical esquerda. */

    switch (i_tipo) {
      case 1: putch (186); break;
      case 2: putch (177); break;
      case 3: putch (186); break;
      case 4: putch (179); break;
      case 5: putch (176); break;
      case 6: putch (179); break;
      case 7: putch (178); break;
      case 8: putch (219); break;
    }

    gotoxy (X2, i_cont);  /* Linha vertical direita. */

    switch (i_tipo) {
      case 1: putch (186); break;
      case 2: putch (177); break;
      case 3: putch (186); break;
      case 4: putch (179); break;
      case 5: putch (176); break;
      case 6: putch (179); break;
      case 7: putch (178); break;
      case 8: putch (219); break;
    }
  }

  for (i_cont = X1 + 1; i_cont < X2; i_cont++) {

    gotoxy  (i_cont, Y1);  /* Linha horizontal superior. */

    switch (i_tipo) {
      case 1: putch (205); break;
      case 2: putch (177); break;
      case 3: putch (196); break;
      case 4: putch (205); break;
      case 5: putch (176); break;
      case 6: putch (196); break;
      case 7: putch (178); break;
      case 8: putch (219); break;
    }

    gotoxy (i_cont, Y2);  /* Linha direita. */

    switch (i_tipo)
    {
      case 1: putch (205); break;
      case 2: putch (177); break;
      case 3: putch (196); break;
      case 4: putch (205); break;
      case 5: putch (176); break;
      case 6: putch (196); break;
      case 7: putch (178); break;
      case 8: putch (219); break;
    }
  }
}

/*--------------------------------------------------------------------------*\
 | tela::escreve_esquerda - escreve no lado esquerdo da tela.               |
 |                                                                          |
 | Recebe - A linha do video para escrever a mensagem e a propria mensagem. |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void tela::escreve_esquerda (const i_linha, const char *cpz_informacao)

{

  gotoxy (3, i_linha);
  cprintf (cpz_informacao);

}


/*--------------------------------------------------------------------------*\
 | tela::escreve_meio - escreve no meio da tela.                            |
 |                                                                          |
 | Recebe - A linha do video para escrever a mensagem e a propria mensagem. |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void tela::escreve_meio (const i_linha, const char *cpz_informacao)

{

  int i_coluna = (80 - strlen (cpz_informacao)) / 2;

  gotoxy (i_coluna, i_linha);
  cprintf (cpz_informacao);

}

/*--------------------------------------------------------------------------*\
 | tela::escreve_direita - escreve no lado direito da tela.                 |
 |                                                                          |
 | Recebe - A linha do video para escrever a mensagem e a propria mensagem. |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void tela::escreve_direita (const i_linha, const char *cpz_informacao)

{

  int i_coluna = 78 - strlen (cpz_informacao);

  gotoxy (i_coluna, i_linha);
  cprintf (cpz_informacao);

}

/*--------------------------------------------------------------------------*\
 | tela::escreve - escreve um string na posicao desejada.                   |
 |                                                                          |
 | Recebe - A coordenada do video, o formato de impressao e o string.       |
 |                                                                          |
 | Retorna - Nada.                                     (sobrecarregada)     |
\*--------------------------------------------------------------------------*/

void tela::escreve (int i_coluna, int i_linha, char *cpz_formato,
		    char *cpz_informacao)
{

  gotoxy (i_coluna, i_linha);
  cprintf (cpz_formato, cpz_informacao);
}

/*--------------------------------------------------------------------------*\
 | tela::escreve - escreve um inteiro na posicao desejada.                  |
 |                                                                          |
 | Recebe - A coordenada do video, o formato de impressao e o inteiro.      |
 |                                                                          |
 | Retorna - Nada.                                     (sobrecarregada)     |
\*--------------------------------------------------------------------------*/
void tela::escreve (int i_coluna, int i_linha, char *cpz_formato,
		    int cpz_informacao)
{

  gotoxy (i_coluna, i_linha);
  cprintf (cpz_formato, cpz_informacao);
}

/*--------------------------------------------------------------------------*\
 | tela::escreve - escreve um long na posicao desejada.                     |
 |                                                                          |
 | Recebe - A coordenada do video, o formato de impressao e o long.         |
 |                                                                          |
 | Retorna - Nada.                                     (sobrecarregada)     |
\*--------------------------------------------------------------------------*/

void tela::escreve (int i_coluna, int i_linha, char *cpz_formato,
		    long int l_informacao)
{

  gotoxy (i_coluna, i_linha);
  cprintf (cpz_formato, l_informacao);
}

/*--------------------------------------------------------------------------*\
 | tela::escreve - escreve um caracter na posicao desejada.                 |
 |                                                                          |
 | Recebe - A coordenada do video, o formato de impressao e o caracter.     |
 |                                                                          |
 | Retorna - Nada.                                     (sobrecarregada)     |
\*--------------------------------------------------------------------------*/

void tela::escreve (int i_coluna, int i_linha, char *cpz_formato,
		    char cpz_informacao)
{

  gotoxy (i_coluna, i_linha);
  cprintf (cpz_formato, cpz_informacao);
}


/*--------------------------------------------------------------------------*\
 | tela::escreve - escreve um float na posicao desejada.                    |
 |                                                                          |
 | Recebe - A coordenada do video, o formato de impressao e o float.        |
 |                                                                          |
 | Retorna - Nada.                                     (sobrecarregada)     |
\*--------------------------------------------------------------------------*/
void tela::escreve (int i_coluna, int i_linha, char *cpz_formato,
		    float cpz_informacao)
{

  gotoxy (i_coluna, i_linha);
  cprintf (cpz_formato, cpz_informacao);
}

/*--------------------------------------------------------------------------*\
 | tela::apaga_mensagem - preenche de epacos o local de mensagens.          |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void tela::apaga_mensagem (void)

{

  gotoxy (2, i_linha_mensagem);
  for (int i = 2; i < 79; i++)
   putch (' ');

}

/*--------------------------------------------------------------------------*\
 | tela::beep - executa um sinal sonoro.                                    |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void tela::beep (void)

{

  sound (500);
  delay (10);
  nosound ();

}

/*--------------------------------------------------------------------------*\
 | tela::preenche_espacos - completa a variavel com espacos ate o seu       |
 |                          tamanho maximo.                                 |
 |                                                                          |
 | Recebe - A variavel a ser modificada e o seu tamanho maximo.             |
 |                                                                          |
 | Retorna - A variavel modificada.                                         |
\*--------------------------------------------------------------------------*/

void tela::preenche_espacos (char *cpz_linha, const int i_tam)

{

  for (int i_cont = strlen (cpz_linha); i_cont < i_tam; i_cont++)
    cpz_linha [i_cont] = ' ';

}

/*--------------------------------------------------------------------------*\
 | tela::edita_linha - executa a edicao de uma linha.                       |
 |                                                                          |
 | Recebe - As coordenadas do video para a edicao, um ponteiro para a linha,|
 |          e o seu tamanho maximo.                                         |
 |                                                                          |
 | Retorna - A linha editada.                                               |
\*--------------------------------------------------------------------------*/

int tela::edita_campo (const int i_coluna, const int i_linha,
		       char *cpz_linha, const int i_tam)
{

  char *cpz_linha_modif = new char [i_tam];

  int i_coluna_cursor = 0;  // localizacao do cursor dentro da linha
  int i_comando;            // comando recebido pelo teclado
  char c_caracter;          // comando de troca de letra

  preenche_espacos (cpz_linha, i_tam); // no final do string

  /* salva a linha anterior */

  for (int i = 0; i < i_tam; i++)
    cpz_linha_modif [i] = cpz_linha [i];


  /* coloca a linha na tela */

  gotoxy (i_coluna, i_linha);
  for (int i_pos = 0; i_pos < i_tam; i_pos++) {
    gotoxy (i_coluna + i_pos, i_linha);
    putch (cpz_linha [i_pos]);
  }
  gotoxy (i_coluna, i_linha);

  do {
    while (bioskey (_KEYBRD_READY) == 0)
    ;

    i_comando = bioskey (_KEYBRD_READ);

    if (isprint (c_caracter = (i_comando & 0xFF))) {
      if (i_coluna_cursor < i_tam) {

	if (c_caracter >= 'a' && c_caracter <= 'z')
	  c_caracter -= 32;

	cpz_linha_modif [i_coluna_cursor] = c_caracter;

	putch (c_caracter);
	i_coluna_cursor++;
	gotoxy (i_coluna + i_coluna_cursor, i_linha);
      }
      else
	beep ();
    }
    else
      switch (i_comando) {

	case (BACKSPACE):

	  if (i_coluna_cursor > 0) {
	    i_coluna_cursor--;
	    gotoxy (i_coluna + i_coluna_cursor, i_linha);
	    putch (' ');
	    cpz_linha_modif [i_coluna_cursor] = ' ';
	    gotoxy (i_coluna + i_coluna_cursor, i_linha);
	  }
	  else
	    beep ();

	  break;

	case (SETA_DIREITA):

	  if (i_coluna_cursor < i_tam) {
	    i_coluna_cursor++;
	    gotoxy (i_coluna + i_coluna_cursor, i_linha);
	  }
	  else
	    beep ();

	  break;

	case (SETA_ESQUERDA):

	  if (i_coluna_cursor > 0) {
	    i_coluna_cursor--;
	    gotoxy (i_coluna + i_coluna_cursor, i_linha);
	  }
	  else
	    beep ();

	  break;

	case (ENTER):
	case (ESC):
	case (SETA_CIMA):
	case (SETA_BAIXO):

	  break;

	default:

	  beep (); // Teclas invalidas como F1, etc
      }

  } while (i_comando != ENTER &&
	   i_comando != ESC &&
	   i_comando != SETA_CIMA &&
	   i_comando != SETA_BAIXO);

  for (int i_posicao_atual = i_tam - 1; // Procura a ultima letra da linha
       i_posicao_atual > 0;
       i_posicao_atual--)
    if (cpz_linha_modif [i_posicao_atual] != ' ')
      break;

  strncpy (cpz_linha, cpz_linha_modif, i_tam);
  cpz_linha [++i_posicao_atual] = NULO;        // Delimita a linha

  delete cpz_linha_modif;

  gotoxy (i_coluna, i_linha);            // Mostra a linha modificada
  for (int i_pos_rep = 0; i_pos_rep < i_posicao_atual; i_pos_rep++) {
    gotoxy (i_coluna + i_pos_rep, i_linha);
    putch (cpz_linha [i_pos_rep]);
  }
  for (int j = i_posicao_atual; j < i_tam; j++) { // Preenche o resto
    gotoxy (i_coluna + j, i_linha);               // com espacos em branco
    putch (' ');
  }

  switch (i_comando) {

    case (ENTER):
    case (SETA_BAIXO):

      return (DESCE);

    case (SETA_CIMA):

      return (SOBE);

  }

  return (SAI);

}

/*--------------------------------------------------------------------------*\
 | tela::pega_numero - executa a edicao de um inteiro.                      |
 |                                                                          |
 | Recebe - As coordenadas do video para a edicao e o tamanho do inteiro.   |
 |          e o seu tamanho maximo.                                         |
 |                                                                          |
 | Retorna - A acao a ser tomada e o inteiro.         (Sobrecarregada)      |
\*--------------------------------------------------------------------------*/

int tela::pega_numero (int i_coluna, int i_linha, int &i_numero,
		       int i_tamanho)
{
  char *cpz_numero = new char [i_tamanho + 1];

  sprintf (cpz_numero, "%d", i_numero);
  preenche_espacos (cpz_numero, i_tamanho + 1);
  int i_retorno = edita_campo (i_coluna, i_linha, cpz_numero, i_tamanho);
  i_numero = atoi (cpz_numero);
  delete cpz_numero;

  return i_retorno;
}

/*--------------------------------------------------------------------------*\
 | tela::pega_numero - executa a edicao de um long.                         |
 |                                                                          |
 | Recebe - As coordenadas do video para a edicao e o tamanho do inteiro.   |
 |          e o seu tamanho maximo.                                         |
 |                                                                          |
 | Retorna - A acao a ser tomada e o long.                                  |
\*--------------------------------------------------------------------------*/

int tela::pega_numero (int i_coluna, int i_linha, long int &l_numero,
		       int i_tamanho)
{
  char *cpz_numero = new char [i_tamanho + 1];

  sprintf (cpz_numero, "%ld", l_numero);
  preenche_espacos (cpz_numero, i_tamanho + 1);
  int l_retorno = edita_campo (i_coluna, i_linha, cpz_numero, i_tamanho);
  l_numero = atol (cpz_numero);
  delete cpz_numero;

  return l_retorno;
}


/*--------------------------------------------------------------------------*\
 | tela::pega_numero - executa a edicao de um float.                        |
 |                                                                          |
 | Recebe - As coordenadas do video para a edicao e o tamanho do inteiro.   |
 |          e o seu tamanho maximo.                                         |
 |                                                                          |
 | Retorna - A acao a ser tomada e o float.                                 |
\*--------------------------------------------------------------------------*/
int tela::pega_numero (int i_coluna, int i_linha, float &f_numero,
		       int i_tamanho, int i_casas_dec)
{
  char *cpz_numero = new char [i_tamanho + i_casas_dec + 1];
  char formato [10], tamanho [10], tamanho_real [10];

  strcpy (formato, "%");
  itoa (i_tamanho, tamanho, 10);
  strcat (tamanho, ".");
  strcat (tamanho, itoa (i_casas_dec, tamanho_real, 10));
  strcat (formato, tamanho);
  strcat (formato, "f");

  sprintf (cpz_numero, formato, f_numero);
  preenche_espacos (cpz_numero, i_tamanho + i_casas_dec + 1);
  int i_retorno = edita_campo (i_coluna, i_linha, cpz_numero,
			       i_tamanho + i_casas_dec + 1);
  f_numero = atof (cpz_numero);
  delete cpz_numero;

  return i_retorno;
}

/*--------------------------------------------------------------------------*\
 | tela::mensagem - envia uma mensagem para a tela e espera continuacao.    |
 |                                                                          |
 | Recebe - Um ponteiro para a mensagem.                                    |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void tela::mensagem (const char *cpz_mensagem)

{
  char *cpz_mensagem_completa = new char [strlen (cpz_mensagem) +
					  strlen (CONTINUAR) + 1];
  /* constroi mensagem */

  strcpy (cpz_mensagem_completa, cpz_mensagem);
  strcat (cpz_mensagem_completa, CONTINUAR);

  escreve_esquerda (i_linha_mensagem, cpz_mensagem_completa);

  getch ();

  apaga_mensagem ();

  delete cpz_mensagem_completa;

}


/*--------------------------------------------------------------------------*\
 | tela::confirmacao - envia um pedido de confirmacao.                      |
 |                                                                          |
 | Recebe - Um ponteiro para a mensagem.                                    |
 |                                                                          |
 | Retorna - A resposta do usuario.                                         |
\*--------------------------------------------------------------------------*/

int tela::confirmacao (const char *cpz_mensagem)

{

  int i_tam_mensagem = strlen (cpz_mensagem) + strlen (CONTINUAR);

  char *cpz_mensagem_completa = new char [i_tam_mensagem + 1];

  char resposta;

  /* constroi mensagem */

  strcpy (cpz_mensagem_completa, cpz_mensagem);
  strcat (cpz_mensagem_completa, CONFIRMACAO);

  escreve_esquerda (i_linha_mensagem, cpz_mensagem_completa);

  do {                               // Recebe confirmacao
    gotoxy (3 + i_tam_mensagem, i_linha_mensagem);
    resposta = getche ();
  } while (resposta != 'S' && resposta != 's' &&
	   resposta != 'N' && resposta != 'n');

  delete cpz_mensagem_completa;

  apaga_mensagem ();

  if (resposta == 'S' || resposta == 's')
    return (SIM);

  return (NAO);

}


/*--------------------------------------------------------------------------*\
 | tela::~tela (destrutor) apaga a tela e retorna ao estado anterios.       |
 |                                                                          |
 | Recebe - Nada                                                            |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/


tela::~tela (void)
  {
    if (cpz_titulo_tela != NULL) {
      delete cpz_titulo_tela;
      cpz_titulo_tela = NULL;
    }

    textmode (sti_modo_ant.currmode);

    clrscr ();
  };
